/*
   ibeacon oder Beliebiger BLE Gerät Präsenz Überwachung mit Relais

   Michael Dworkin http://esp32-server.de/
   Based on Neil Kolban example for IDF: https://github.com/nkolban/esp32-snippets/blob/master/cpp_utils/tests/BLE%20Tests/SampleScan.cpp
   Ported to Arduino ESP32 by Evandro Copercini
*/

#include <BLEDevice.h>
#include <BLEUtils.h>
#include <BLEScan.h>
#include <BLEAdvertisedDevice.h>
#include <BLEAddress.h>
#include <Ticker.h>

#include <WiFi.h>
#include <WiFiClient.h>
#include <WebServer.h>
#include <ESPmDNS.h>
#include <SPIFFS.h>
#include <string.h>
#include <FS.h>

#define debug true
#define SetupPin 13   // INI wifimanager starten mit LOW-Pegel an SetupPin

WebServer server(80);
#include "INIwifimanager.h"

String MACadresse = "00:15:83:40:7c:bb"; // Bluetooth Adresse die überwacht wird
int RelaisPin = 22;             // Pin an dem Relais hängt
int Verzoegerung = 15;                // Auschaltverzögerung wenn das Signal von BLE ibeacon fehlt
int active_low = 1;   //  "0" Relais schaltet bei +5V, "1" Relais beim schaltet bei 0V
int MinRSSI =-120;

int VerzoegerungZaeler = 0;
Ticker Tic;
static BLEAddress *pServerAddress;
BLEScan* pBLEScan ;
int scanTime = 10; //In seconds



class MyAdvertisedDeviceCallbacks: public BLEAdvertisedDeviceCallbacks
{
    void onResult(BLEAdvertisedDevice advertisedDevice) // passiert wenn BLE Device ( beacon ) gefunden wurde
    {
      Serial.print(advertisedDevice.getAddress().toString().c_str()); // ibeacon Adresse anzeigen
      Serial.printf("   RSSI: %d ", advertisedDevice.getRSSI());
      if (advertisedDevice.haveServiceUUID())
      {
        Serial.printf("UUID: %s ", advertisedDevice.getServiceUUID().toString().c_str());
      }
      if (advertisedDevice.getAddress().equals(*pServerAddress) & advertisedDevice.getRSSI() > MinRSSI)    // ibeacon Adresse Vergleichen
      {
        Serial.print(" Ueberwachte Adresse");                         // wenn überwache Adresse gefunden wurde
        digitalWrite (RelaisPin, !active_low);                                  // Relais Einschalten
        VerzoegerungZaeler = 0;                                       // Ausschaltverzögerung zurücksetzen
        advertisedDevice.getScan()->stop();                           // Scanvorgang beenden
      } // Found our server
      Serial.println("");
    }
};


void SekundenTic()  // Wird jede Sekunde ausgefüert
{
  VerzoegerungZaeler++;  // Sekundenzähler
  if (VerzoegerungZaeler >= Verzoegerung) digitalWrite (RelaisPin, active_low); // Wenn Verzögerungszeit erreicht wurde Auschalten
}

//******************************************************************************
//  Diese Funktion wird beim lesen eder Zeile der INI-Datei ausgeführt         *
//******************************************************************************

void DatenLaden (String argument, String Wert)
{
  if (argument.indexOf ( "macadresse" ) >= 0)  MACadresse = Wert;
  if (argument.indexOf ( "verzoegerung" ) >= 0)   Verzoegerung = Wert.toInt();
  if (argument.indexOf ( "relaispin" ) >= 0)   RelaisPin = Wert.toInt();
  if (argument.indexOf ( "active_low" ) >= 0)   active_low = Wert.toInt();
  if (argument.indexOf ( "rssi" ) >= 0)   MinRSSI= Wert.toInt();
}

void setup()
{
  pinMode (RelaisPin, OUTPUT);
  digitalWrite (RelaisPin, active_low);
  pinMode(SetupPin, INPUT_PULLUP);       // starte INI WiFiManager wenn SetupPin Low ist
  char inser;
  Serial.begin(115200);
  Serial.println("");
  if (!INI_Einlesen()) Einstellen();     // INI -Datei einlesen, wenn geht nicht, starte INI WiFiManager

  //******************************************************************************
  //         Setup Weiche, Erkennen ob TXD/RXD gebrückt sind                     *
  //******************************************************************************
  while (Serial.available())inser = Serial.read();  // RX - Buffer leern
  Serial.println("Start");
  delay(10);
  inser = Serial.read();
  if (inser != 'S') goto weiter;
  inser = Serial.read();
  if (inser != 't') goto weiter;
  Einstellen();         // starte INI WiFiManager  wenn TXD / RXD verbunden sind
weiter:
  if (!digitalRead(SetupPin)) Einstellen();

  Serial.println("Starte BLE Scanner");
  pServerAddress = new BLEAddress(MACadresse.c_str());
  BLEDevice::init("");
  pBLEScan = BLEDevice::getScan(); //create new scan
  pBLEScan->setAdvertisedDeviceCallbacks(new MyAdvertisedDeviceCallbacks());
  pBLEScan->setActiveScan(true); //active scan uses more power, but get results faster
  Tic.attach( 1, SekundenTic);
  pBLEScan->start(scanTime);
}


void loop()
{
  pBLEScan->start(scanTime);
  delay(100);      
}
